""" 
    Copyright (C) 2009 Sam Vesurai (sam94089@gmail.com).

    pySpykeeWeb is a simple web server for controlling a Spykee robot.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation version 3 of the License.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

from twisted.internet import gtk2reactor
gtk2reactor.install()

from twisted.internet import reactor, protocol,defer
from twisted.web import http,  server,  resource

# this implements spykee protocol
import twistedprotocol
import ConfigParser

class WebNavForm(resource.Resource):
        isLeaf = True

	def __init__(self, streamip):
		resource.Resource.__init__(self) 
		self.streamip = streamip

        def render(self, request):
                cmd = request.args.get('command', '')
                
                if len(cmd)>0:
			if cmd[0]=="StartVideo":
                                SpykeeControl.clientFactory.currentProtocol.activateVideo()
                        if cmd[0]=="Undock":
                                SpykeeControl.clientFactory.currentProtocol.undock()
                                SpykeeControl.clientFactory.currentProtocol.activateVideo()
                        if cmd[0]=="Dock":
                                SpykeeControl.clientFactory.currentProtocol.dock()
                        if cmd[0]=="Forward":
                                SpykeeControl.clientFactory.currentProtocol.motorForward(80,1)
                        if cmd[0]=="Back":
                                SpykeeControl.clientFactory.currentProtocol.motorBack(80,1)
                        if cmd[0]=="Left":
                                SpykeeControl.clientFactory.currentProtocol.motorLeft(0.2)
                        if cmd[0]=="Right":
                                SpykeeControl.clientFactory.currentProtocol.motorRight(0.2)
                
                form = """
                 <html><body>
                 <FORM action="/" method="post">
                <BUTTON name="command" style=width:50 value="Forward" type="submit">FWD</button>
                <BUTTON name="command" style=width:50 value="Back" type="submit">BACK</button>
                <BUTTON name="command" style=width:50 value="Left" type="submit">LEFT</button>
                <BUTTON name="command" style=width:50 value="Right" type="submit">RIGHT</button>
		<br>
                <BUTTON name="command" value="StartVideo" type="submit">Start Video</button>
                <BUTTON name="command" value="Undock" type="submit">UNDOCK</button>
                <BUTTON name="command" value="Dock" type="submit">DOCK</button>
                </form>
               <img src="http://%s:8000">
		<br>Battery Level:%d</body></html>
                """ % (self.streamip,SpykeeControl.batterylevel)
                return form
        
def wait(seconds, result=None):
    d = defer.Deferred()
    reactor.callLater(seconds, d.callback, result)
    return d

class StreamHandler(http.Request):
    BOUNDARY = 'BOUNDARY'

    def writeBoundary(self):
        self.write("--%s\n" % (self.BOUNDARY))

    @defer.inlineCallbacks
    def process(self):
        self.setHeader('Connection', 'Keep-Alive')
        self.setHeader('Content-Type', "multipart/x-mixed-replace;boundary=%s" % (self.BOUNDARY))
        self.setHeader('Pragma','no-cache')
        self.setHeader('Cache-Control','no-cache')

        self.writeBoundary()
        while True:
                self.write("Content-Type: image/jpeg\n")
                self.write("Content-Length: %s\n\n" % (len(SpykeeControl.currentFrame)))
                self.write(SpykeeControl.currentFrame)
                self.writeBoundary()
                yield wait(0.5)

class StreamProtocol(http.HTTPChannel):
    requestFactory = StreamHandler
class StreamFactory(http.HTTPFactory):
    protocol = StreamProtocol
    
class SpykeeControl():
    currentFrame = "1"
    batterylevel = None
    clientFactory = None
    
    def __init__(self, robot_name,  robot_ip, username, password):
        self.username = username
        self.password = password
        print "connecting to %s (%s) with %s:%s" % (robot_name, robot_ip, username, password)
        self.cf = twistedprotocol.SpykeeClientFactory(self.username, self.password, self)
        SpykeeControl.clientFactory = self.cf
        reactor.connectTCP(robot_ip, 9000, self.cf)

    # protocol callbacks
    def isDocked(self, docked):
        SpykeeControl.clientFactory.currentProtocol.setSoundVolume(10)
        pass
    def connectionLost(self, reason):
        print "Connection lost!"
    def connectionFailed(self, reason):
        print "Failed to connect to Spykee!"
    
    def videoFrame(self, frame):
        SpykeeControl.currentFrame = frame

    def audioSample(self, sample):
        pass

    def batteryLevel(self, level):
	SpykeeControl.batterylevel = level


class SpykeeWebConfig():

	username = None
	password = None
	streamip = None

	def getPublicIP(self):
		import urllib
		return urllib.urlopen('http://whatismyip.org').read()

	def getLocalIP(self):
		import netifaces
		for a in netifaces.interfaces():
			ip = netifaces.ifaddresses(a)
			if ip.has_key(netifaces.AF_INET):
				if not ip[netifaces.AF_INET][0]['addr'].startswith('127.'):
					return ip[netifaces.AF_INET][0]['addr']
		return None

	def __init__(self):
        	from pkg_resources import resource_filename
		self.configfile = resource_filename(__name__, 'pySpykeeWeb.conf')        
		self.config = ConfigParser.RawConfigParser()
		self.config.read(self.configfile)
		
		self.username = self.config.get('pyspykeeweb','user')
		print "username = %s" % self.username
		self.password = self.config.get('pyspykeeweb','password')
		print "password = %s" % self.password
		
		# default streaming mode is local
		if self.config.get('pyspykeeweb','mode')=='local':
			self.streamip = self.getLocalIP()
		else:	
			self.streamip = self.getPublicIP()

		print "streaming IP address = %s" % self.streamip

def discovered(spykees):
        if not spykees:
                print "No Spykee rebot found."
                reactor.stop()
                return False
               
        print "Found %s" % spykees
	conf = SpykeeWebConfig()

	# sometimes spykee returns a bad username, gotta fix it
	i = spykees.keys()[0].find(',')
	if i>0:
		botname = spykees.keys()[0][:i]
	else:
		botname = spykees.keys()[0]
        SpykeeControl(botname, spykees[spykees.keys()[0]], conf.username, conf.password)
        reactor.listenTCP(8001,  server.Site(WebNavForm(conf.streamip)))
        reactor.listenTCP(8000, StreamFactory())

def main():
	print "\npySpykeeWeb Copyright (C) 2009 Sam Vesurai (sam94089@gmail.com)."
	print "This program comes with ABSOLUTELY NO WARRANTY;"
	print "This is free software, and you are welcome to redistribute it under certain conditions.\n"

        d = twistedprotocol.discover(5)
        r = d.addCallback(discovered)
	if r:
        	reactor.run()
    
if __name__ == '__main__':
        main()
